@@ -30,6 +30,8 @@
 #ifndef __MATH__
 #define __MATH__
 
+#include "sys/cdefs.h" /* For definition of __DARWIN_UNIX03 et al */
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -38,17 +40,32 @@ extern "C" {
 *       Floating point data types                                             *
 ******************************************************************************/
 
-/* ix87 FPU evaluates all quantities in the 80-bit long double format. */
-typedef long double	float_t;
-typedef long double	double_t;
+/*	Define float_t and double_t per C standard, ISO/IEC 9899:1999 7.12 2,
+	taking advantage of GCC's __FLT_EVAL_METHOD__ (which a compiler may
+	define anytime and GCC does) that shadows FLT_EVAL_METHOD (which a compiler
+	must and may define only in float.h).
+*/
+#if __FLT_EVAL_METHOD__ == 0
+	typedef float float_t;
+	typedef double double_t;
+#elif __FLT_EVAL_METHOD__ == 1
+	typedef double float_t;
+	typedef double double_t;
+#elif __FLT_EVAL_METHOD__ == 2 || __FLT_EVAL_METHOD__ == -1
+	typedef long double float_t;
+	typedef long double double_t;
+#else /* __FLT_EVAL_METHOD__ */
+	#error "Unsupported value of __FLT_EVAL_METHOD__."
+#endif /* __FLT_EVAL_METHOD__ */
+
 
 #define	HUGE_VAL	1e500
 #define	HUGE_VALF	1e50f
-#define	HUGE_VALL	1e5000
+#define	HUGE_VALL	1e5000L
 
 #define INFINITY	HUGE_VALF
 
-#if defined(__APPLE_CC__) && (__APPLE_CC__ >= 1345)
+#if defined(__GNUC__)
 #define NAN		__builtin_nanf("0x7fc00000") /* Constant expression, can be used as initializer. */
 #else
 #define NAN		__nan( )
@@ -59,18 +76,31 @@ typedef long double	double_t;
 ******************************************************************************/
 
 enum {
-	FP_NAN          = 1,                   /*      NaN                    */
-	FP_INFINITE     = 2,                   /*      + or - infinity        */
-	FP_ZERO         = 3,                   /*      + or - zero            */
-	FP_NORMAL       = 4,                   /*      all normal numbers     */
-	FP_SUBNORMAL    = 5                    /*      denormal numbers       */
+	_FP_NAN          = 1,                   /*      NaN                    */
+	_FP_INFINITE     = 2,                   /*      + or - infinity        */
+	_FP_ZERO         = 3,                   /*      + or - zero            */
+	_FP_NORMAL       = 4,                   /*      all normal numbers     */
+	_FP_SUBNORMAL    = 5,					/*      denormal numbers       */
+	_FP_SUPERNORMAL  = 6                    /*      long double delivering > LDBL_DIG, e.g. 1. + 2^-1000 */
 };
 
-/* "Fused" multiply-add is not supported on Intel as a single fast opcode. */
-#define FP_FAST_FMA	0
-#define FP_FAST_FMAF	0
+#define FP_NAN          _FP_NAN
+#define FP_INFINITE     _FP_INFINITE
+#define FP_ZERO         _FP_ZERO
+#define FP_NORMAL       _FP_NORMAL
+#define FP_SUBNORMAL    _FP_SUBNORMAL
+#define FP_SUPERNORMAL  _FP_SUPERNORMAL
 
-/* The values returned by `ilogb' for 0 and NaN respectively.  */
+/* fma() *function call* is more costly than equivalent (in-line) multiply and add operations    */
+/* For single and double precision, the cost isn't too bad, because we can fall back on higher   */
+/* precision hardware, with the necessary range to handle infinite precision products. However,  */
+/* expect the long double fma to be at least an order of magnitude slower than a simple multiply */
+/* and an add.                                                                                   */
+#undef FP_FAST_FMA
+#undef FP_FAST_FMAF
+#undef FP_FAST_FMAL
+
+/* The values returned by `ilogb' for 0 and NaN respectively. */
 #define FP_ILOGB0	(-2147483647 - 1)
 #define FP_ILOGBNAN	(-2147483647 - 1)
 
@@ -94,108 +124,59 @@ extern unsigned int __math_errhandling ( void );
 *                                                                               *
 ********************************************************************************/
 
-#define      fpclassify( x )    ( ( sizeof ( x ) == sizeof(double) ) ?           \
-                              __fpclassifyd  ( x ) :                           \
-                                ( sizeof ( x ) == sizeof( float) ) ?            \
-                              __fpclassifyf ( x ) :                            \
-                              __fpclassify  ( x ) )
-#define      isnormal( x )      ( ( sizeof ( x ) == sizeof(double) ) ?           \
-                              __isnormald ( x ) :                              \
-                                ( sizeof ( x ) == sizeof( float) ) ?            \
-                              __isnormalf ( x ) :                              \
-                              __isnormal  ( x ) )
-#define      isfinite( x )      ( ( sizeof ( x ) == sizeof(double) ) ?           \
-                              __isfinited ( x ) :                              \
-                                ( sizeof ( x ) == sizeof( float) ) ?            \
-                              __isfinitef ( x ) :                              \
-                              __isfinite  ( x ) )
-#define      isinf( x )         ( ( sizeof ( x ) == sizeof(double) ) ?           \
-                              __isinfd ( x ) :                                 \
-                                ( sizeof ( x ) == sizeof( float) ) ?            \
-                              __isinff ( x ) :                                 \
-                              __isinf  ( x ) )
-#define      isnan( x )         ( ( sizeof ( x ) == sizeof(double) ) ?           \
-                              __isnand ( x ) :                                 \
-                                ( sizeof ( x ) == sizeof( float) ) ?            \
-                              __isnanf ( x ) :                                 \
-                              __isnan  ( x ) )
-#define      signbit( x )       ( ( sizeof ( x ) == sizeof(double) ) ?           \
-                              __signbitd ( x ) :                               \
-                                ( sizeof ( x ) == sizeof( float) ) ?            \
-                              __signbitf ( x ) :                               \
-                              __signbitl  ( x ) )
+#define fpclassify(x)	\
+	(	sizeof (x) == sizeof(float )	?	__fpclassifyf(x)	\
+	:	sizeof (x) == sizeof(double)	?	__fpclassifyd(x)	\
+										:	__fpclassify (x))
 
-#define __TYPE_LONGDOUBLE_IS_DOUBLE 0
-                              
-extern long  __fpclassifyd( double );
-extern long  __fpclassifyf( float );
-extern long  __fpclassify( long double );
-#if __TYPE_LONGDOUBLE_IS_DOUBLE
-#ifdef __cplusplus
-    inline long __fpclassify( long double x ) { return __fpclassifyd((double)( x )); }
-#else
-#define __fpclassify( x ) (__fpclassifyd((double)( x )))
-#endif
-#endif
+#define isnormal(x)	\
+	(	sizeof (x) == sizeof(float )	?	__isnormalf(x)	\
+	:	sizeof (x) == sizeof(double)	?	__isnormald(x)	\
+										:	__isnormal (x))
 
+#define isfinite(x)	\
+	(	sizeof (x) == sizeof(float )	?	__isfinitef(x)	\
+	:	sizeof (x) == sizeof(double)	?	__isfinited(x)	\
+										:	__isfinite (x))
 
-extern long  __isnormald( double );
-extern long  __isnormalf( float );
-extern long  __isnormal( long double );
-#if __TYPE_LONGDOUBLE_IS_DOUBLE
-#ifdef __cplusplus
-    inline long __isnormal( long double x ) { return __isnormald((double)( x )); }
-#else
-#define __isnormal( x ) (__isnormald((double)( x )))
-#endif
-#endif
+#define isinf(x)	\
+	(	sizeof (x) == sizeof(float )	?	__isinff(x)	\
+	:	sizeof (x) == sizeof(double)	?	__isinfd(x)	\
+										:	__isinf (x))
 
-extern long  __isfinited( double );
-extern long  __isfinitef( float );
-extern long  __isfinite( long double );
-#if __TYPE_LONGDOUBLE_IS_DOUBLE
-#ifdef __cplusplus
-    inline long __isfinite( long double x ) { return __isfinited((double)( x )); }
-#else
-#define __isfinite( x ) (__isfinited((double)( x )))
-#endif
-#endif
+#define isnan(x)	\
+	(	sizeof (x) == sizeof(float )	?	__isnanf(x)	\
+	:	sizeof (x) == sizeof(double)	?	__isnand(x)	\
+										:	__isnan (x))
 
+#define signbit(x)	\
+	(	sizeof (x) == sizeof(float )	?	__signbitf(x)	\
+	:	sizeof (x) == sizeof(double)	?	__signbitd(x)	\
+										:	__signbitl(x))
 
-extern long  __isinfd( double );
-extern long  __isinff( float );
-extern long  __isinf( long double );
-#if __TYPE_LONGDOUBLE_IS_DOUBLE
-#ifdef __cplusplus
-    inline long __isinf( long double x ) { return __isinfd((double)( x )); }
-#else
-#define __isinf( x ) (__isinfd((double)( x )))
-#endif
-#endif
+extern int __fpclassifyf(float      );
+extern int __fpclassifyd(double     );
+extern int __fpclassify (long double);
 
+extern int __isnormalf  (float      );
+extern int __isnormald  (double     );
+extern int __isnormal   (long double);
 
-extern long  __isnand( double );
-extern long  __isnanf( float );
-extern long  __isnan( long double );
-#if __TYPE_LONGDOUBLE_IS_DOUBLE
-#ifdef __cplusplus
-    inline long __isnan( long double x ) { return __isnand((double)( x )); }
-#else
-#define __isnan( x ) (__isnand((double)( x )))
-#endif
-#endif
+extern int __isfinitef  (float      );
+extern int __isfinited  (double     );
+extern int __isfinite   (long double);
 
+extern int __isinff     (float      );
+extern int __isinfd     (double     );
+extern int __isinf      (long double);
 
-extern long  __signbitd( double );
-extern long  __signbitf( float );
-extern long  __signbitl( long double );
-#if __TYPE_LONGDOUBLE_IS_DOUBLE
-#ifdef __cplusplus
-    inline long __signbitl( long double x ) { return __signbitd((double)( x )); }
-#else
-#define __signbitl( x ) (__signbitd((double)( x )))
-#endif
-#endif
+extern int __isnanf     (float      );
+extern int __isnand     (double     );
+extern int __isnan      (long double);
+
+extern int __signbitf   (float      );
+extern int __signbitd   (double     );
+extern int __signbitl   (long double);
 
 /********************************************************************************
 *                                                                               *
@@ -371,79 +352,134 @@ extern float fminf ( float, float );
 extern double fma ( double, double, double );
 extern float fmaf ( float, float, float );
 
-#define isgreater(x, y) __builtin_isgreater (x, y)
-#define isgreaterequal(x, y) __builtin_isgreaterequal (x, y)
-#define isless(x, y) __builtin_isless (x, y)
-#define islessequal(x, y) __builtin_islessequal (x, y)
-#define islessgreater(x, y) __builtin_islessgreater (x, y)
-#define isunordered(x, y) __builtin_isunordered (x, y)
+extern long double acosl(long double);
+extern long double asinl(long double);
+extern long double atanl(long double);
+extern long double atan2l(long double, long double);
+extern long double cosl(long double);
+extern long double sinl(long double);
+extern long double tanl(long double);
+extern long double acoshl(long double);
+extern long double asinhl(long double);
+extern long double atanhl(long double);
+extern long double coshl(long double);
+extern long double sinhl(long double);
+extern long double tanhl(long double);
+extern long double expl(long double);
+extern long double exp2l(long double);
+extern long double expm1l(long double);
+extern long double logl(long double);
+extern long double log10l(long double);
+extern long double log2l(long double);
+extern long double log1pl(long double);
+extern long double logbl(long double);
+extern long double modfl(long double, long double *);
+extern long double ldexpl(long double, int);
+extern long double frexpl(long double, int *);
+extern int ilogbl(long double);
+extern long double scalbnl(long double, int);
+extern long double scalblnl(long double, long int);
+extern long double fabsl(long double);
+extern long double cbrtl(long double);
+extern long double hypotl(long double, long double);
+extern long double powl(long double, long double);
+extern long double sqrtl(long double);
+extern long double erfl(long double);
+extern long double erfcl(long double);
+extern long double lgammal(long double);
+extern long double tgammal(long double);
+extern long double ceill(long double);
+extern long double floorl(long double);
+extern long double nearbyintl(long double);
+extern long double rintl(long double);
+extern long int lrintl(long double);
+extern long long int llrintl(long double);
+extern long double roundl(long double);
+extern long int lroundl(long double);
+extern long long int llroundl(long double);
+extern long double truncl(long double);
+extern long double fmodl(long double, long double);
+extern long double remainderl(long double, long double);
+extern long double remquol(long double, long double, int *);
+extern long double copysignl(long double, long double);
+extern long double nanl(const char *);
+extern long double nextafterl(long double, long double);
+extern double nexttoward(double, long double);
+extern float nexttowardf(float, long double);
+extern long double nexttowardl(long double, long double);
+extern long double fdiml(long double, long double);
+extern long double fmaxl(long double, long double);
+extern long double fminl(long double, long double);
+extern long double fmal(long double, long double, long double);
+
+#define isgreater(x, y) __builtin_isgreater ((x),(y))
+#define isgreaterequal(x, y) __builtin_isgreaterequal ((x),(y))
+#define isless(x, y) __builtin_isless ((x),(y))
+#define islessequal(x, y) __builtin_islessequal ((x),(y))
+#define islessgreater(x, y) __builtin_islessgreater ((x),(y))
+#define isunordered(x, y) __builtin_isunordered ((x),(y))
 
 extern double  		__inf( void );
 extern float  		__inff( void );
 extern long double  	__infl( void );
 extern float  		__nan( void ); /* 10.3 (and later) must retain in ABI for backward compatability */
 
-#ifndef __NOEXTENSIONS__
+#if !defined(_ANSI_SOURCE)
+extern double j0 ( double );
+
+extern double j1 ( double );
+
+extern double jn ( int, double );
+
+extern double y0 ( double );
+
+extern double y1 ( double );
+
+extern double yn ( int, double );
+
+extern double scalb ( double, double ); 
+
+
+#define M_E         2.71828182845904523536028747135266250   /* e */
+#define M_LOG2E     1.44269504088896340735992468100189214   /* log 2e */
+#define M_LOG10E    0.434294481903251827651128918916605082  /* log 10e */
+#define M_LN2       0.693147180559945309417232121458176568  /* log e2 */
+#define M_LN10      2.30258509299404568401799145468436421   /* log e10 */
+#define M_PI        3.14159265358979323846264338327950288   /* pi */
+#define M_PI_2      1.57079632679489661923132169163975144   /* pi/2 */
+#define M_PI_4      0.785398163397448309615660845819875721  /* pi/4 */
+#define M_1_PI      0.318309886183790671537767526745028724  /* 1/pi */
+#define M_2_PI      0.636619772367581343075535053490057448  /* 2/pi */
+#define M_2_SQRTPI  1.12837916709551257389615890312154517   /* 2/sqrt(pi) */
+#define M_SQRT2     1.41421356237309504880168872420969808   /* sqrt(2) */
+#define M_SQRT1_2   0.707106781186547524400844362104849039  /* 1/sqrt(2) */
+
+#define	MAXFLOAT	((float)3.40282346638528860e+38)
+extern int signgam;     /* required for unix 2003 */
+
+
+#endif /* !defined(_ANSI_SOURCE) */
+
+#if !defined(__NOEXTENSIONS__) && !defined(_POSIX_C_SOURCE)
+#define __WANT_EXTENSIONS__
+#endif
+
+#ifdef __WANT_EXTENSIONS__
 
 #define FP_SNAN		FP_NAN
 #define FP_QNAN		FP_NAN
 
-extern long int rinttol ( double );
+extern long int rinttol ( double );		/* Legacy API: please use C99 lrint() instead. */
 
-extern long int roundtol ( double );
-
-typedef struct __complex_s {
-        double Real;
-        double Imag;
-} __complex_t;
-
-typedef struct __complexf_s {
-        float Real;
-        float Imag;
-} __complexf_t;
+extern long int roundtol ( double );	/* Legacy API: please use C99 lround() instead. */
 
 /*
  * XOPEN/SVID
  */
-#if !defined(_ANSI_SOURCE) && !defined(_POSIX_SOURCE)
-#define	M_E		2.7182818284590452354	/* e */
-#define	M_LOG2E		1.4426950408889634074	/* log 2e */
-#define	M_LOG10E	0.43429448190325182765	/* log 10e */
-#define	M_LN2		0.69314718055994530942	/* log e2 */
-#define	M_LN10		2.30258509299404568402	/* log e10 */
-#define	M_PI		3.14159265358979323846	/* pi */
-#define	M_PI_2		1.57079632679489661923	/* pi/2 */
-#define	M_PI_4		0.78539816339744830962	/* pi/4 */
-#define	M_1_PI		0.31830988618379067154	/* 1/pi */
-#define	M_2_PI		0.63661977236758134308	/* 2/pi */
-#define	M_2_SQRTPI	1.12837916709551257390	/* 2/sqrt(pi) */
-#define	M_SQRT2		1.41421356237309504880	/* sqrt(2) */
-#define	M_SQRT1_2	0.70710678118654752440	/* 1/sqrt(2) */
-
-#define	MAXFLOAT	((float)3.40282346638528860e+38)
-extern int signgam;
-
+#if !defined(_ANSI_SOURCE) && !defined(_POSIX_C_SOURCE)
 #if !defined(_XOPEN_SOURCE)
-enum fdversion {fdlibm_ieee = -1, fdlibm_svid, fdlibm_xopen, fdlibm_posix};
-
-#define _LIB_VERSION_TYPE enum fdversion
-#define _LIB_VERSION _fdlib_version  
-
-/* if global variable _LIB_VERSION is not desirable, one may 
- * change the following to be a constant by: 
- *	#define _LIB_VERSION_TYPE const enum version
- * In that case, after one initializes the value _LIB_VERSION (see
- * s_lib_version.c) during compile time, it cannot be modified
- * in the middle of a program
- */ 
-extern  _LIB_VERSION_TYPE  _LIB_VERSION;
-
-#define _IEEE_  fdlibm_ieee
-#define _SVID_  fdlibm_svid
-#define _XOPEN_ fdlibm_xopen
-#define _POSIX_ fdlibm_posix
-
 #if !defined(__cplusplus)
+/* used by matherr below */
 struct exception {
 	int type;
 	char *name;
@@ -470,38 +506,15 @@ struct exception {
 #define	PLOSS		6
 
 #endif /* !_XOPEN_SOURCE */
-#endif /* !_ANSI_SOURCE && !_POSIX_SOURCE */
+#endif /* !_ANSI_SOURCE && !_POSIX_C_SOURCE */
 
-#if !defined(_ANSI_SOURCE) && !defined(_POSIX_SOURCE)
-extern int finite ( double );
-extern int finitef ( float );
+#if !defined(_ANSI_SOURCE) && !defined(_POSIX_C_SOURCE)
+extern int finite ( double );			/* Legacy API: please use C99 isfinite() instead. */
 
-extern double gamma ( double );
-extern float gammaf ( float );
-
-extern double j0 ( double );
-extern float j0f ( float );
-
-extern double j1 ( double );
-extern float j1f ( float );
-
-extern double jn ( int, double );
-extern float jnf ( int, float );
-
-extern double y0 ( double );
-extern float y0f ( float );
-
-extern double y1 ( double );
-extern float y1f ( float );
-
-extern double yn ( int, double );
-extern float ynf ( int, float );
+extern double gamma ( double );			/* Legacy API: please use C99 tgamma() instead. */
 
 #if !defined(_XOPEN_SOURCE)
 
-extern double scalb ( double, int );
-extern float scalbf ( float, int );
-
 #if !defined(__cplusplus)
 extern int matherr ( struct exception * );
 #endif
@@ -514,31 +527,12 @@ extern double significand ( double );
 /*
  * BSD math library entry points
  */
-#ifndef __COMPLEX__
-#define complex _Complex
-extern double cabs ( double complex );
-extern float cabsf ( float complex );
-#undef complex
-#endif
+extern double drem ( double, double );	/* Legacy API: please use C99 remainder() instead. */
 
-extern double drem ( double, double );
-extern float dremf ( float, float );
-
-/*
- * Reentrant version of gamma & lgamma; passes signgam back by reference
- * as the second argument; user must allocate space for signgam.
- */
-#ifdef _REENTRANT
-extern double gamma_r ( double, int * );
-extern float gammaf_r ( float, int * );
-
-extern double lgamma_r ( double, int * );
-extern float lgammaf_r ( float, int * );
-#endif /* _REENTRANT */
 #endif /* !_XOPEN_SOURCE */
-#endif /* !_ANSI_SOURCE && !_POSIX_SOURCE */
+#endif /* !_ANSI_SOURCE && !_POSIX_C_SOURCE */
 
-#endif /* __NOEXTENSIONS__ */
+#endif /* __WANT_EXTENSIONS__ */
 
 #ifdef __cplusplus
 }
